#!/usr/bin/perl 

############## LICENSE
########################################

# Copyright (C) 2013 Branislav Mitterpach


# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>


########################################
############## ENF OF LICENSE




package main;

use warnings;
use strict;
use Data::Dumper;
use Time::Local;
use Net::SMTP::SSL;
use Cwd 'abs_path';

my $start_time = time();

############## PROCESSING OF PATH.CONF CONFIGURATION FILE
#====================================================================

my $log_folder;
my $install_folder;
my $reports_folder;
my $database_path;
my $leases_path;
our $own_log_path;


my @path_array = split( /\//, abs_path($0) );
pop(@path_array);

my $path_path='';
foreach my $part (@path_array)
{
$path_path=$path_path.$part.'/';
}

open( my $conf, "$path_path"."paths.conf" )
  or die
  "Could not open paths configuration file, no such file '$path_path"."paths.conf' \n";
while (<$conf>) {

    if ( $_ !~ m{^#.*|^\s} ) {
        my @array = split( /\s+/, $_ );

        if ( $array[0] eq 'log_folder' ) {
            $log_folder = $array[1];
        }
        if ( $array[0] eq 'install_folder' ) {
            $install_folder = $array[1];
        }
        if ( $array[0] eq 'reports_folder' ) {
            $reports_folder = $array[1];
        }
        if ( $array[0] eq 'database_path' ) {
            $database_path = $array[1];
        }
        if ( $array[0] eq 'leases_path' ) {
            $leases_path = $array[1];
        }
        if ( $array[0] eq 'own_log_path' ) {
            $own_log_path = $array[1];
        }

    }

}
close($conf) or die "Could not close configuration file 'paths.conf' \n";

#====================================================================
############## END OF PROCESSING

logging(
    "--------------START OF APPLICATION--------------
		Gathered information from configuration file :
		log_folder '$log_folder'
		install_folder '$install_folder'
		reports_folder '$reports_folder'
		database_path '$database_path'
		leases_path '$leases_path'
		own_log_path '$own_log_path'"
);

################ PROCESSING OF ARGUMENTS
#-------------------------------------------------------
our $argEMAIL = 0;
my $argWEB      = 0;
my $argADDRESS  = 0;
my $argNOreport = 0;
my $argIP       = 0;
my $argURL      = 0;
my $argFILE     = 0;
my $argDEFAULT  = 0;
my $argTIMElow  = 0;
my $argTIMEhigh = 0;
my $argLASTdays=0;

if ( !@ARGV ) {
	$argDEFAULT = 1;
	logging(" No arguments detected \@ARGV = '@ARGV' ");

}

if (@ARGV) {

	while (@ARGV) {
		my $arg       = shift(@ARGV);
		my $wrong_arg = $arg;
		if ( $arg eq '-h' ) {
			basic_help();
			logging("Help invoked");
			logging("Aplication ending normally");
			logging("----------------END OF APPLICATION--------------");
			die("\n");
		}
		
		if ( $arg eq '-t' ) {
			$arg = shift(@ARGV);
			logging("Argument -t detected");
			
			if ( $arg =~m{^\d{4}-\d{2}-\d{2}T{0,1}\d{0,2}:{0,1}\d{0,2}:{0,1}\d{0,2}$} )
			{
				$argTIMElow = $arg;
				logging("Start time \(Lower bound\) detected in correct form '$argTIMElow' ");
				
				if(scalar(@ARGV))
				{
				$arg = shift(@ARGV);
				}
				else
				{
				$arg=1;
				}

				if ( $arg =~m{^\d{4}-\d{2}-\d{2}T{0,1}\d{0,2}:{0,1}\d{0,2}:{0,1}\d{0,2}$})
				{
					$argTIMEhigh = $arg;
					logging("End time \(Higher bound\) detected in correct form '$argTIMEhigh' ");
				}
				else
				{
					$argTIMEhigh=GETtimestamp(time());
					logging("End time \(Higher bound\) not detected, seting actual '$argTIMEhigh' time as end time");
				}
			}
            else {
			 logging("Wrong time format in form '$arg' \n");logend();
                die "Wrong time format in form '$arg' \n";
            }
        }

        if ( $arg =~ m{(^\-[eawn]{1,4})} ) {

            if ( $arg =~ m{(.*e.*)} ) {
                $argEMAIL = 1;
                logging("Argument -e detected");
            }

            if ( $arg =~ m{(.*a.*)} ) {
                $argADDRESS = 1;
                logging("Argument -a detected");
            }

            if ( $arg =~ m{(.*w.*)} ) {
                $argWEB = 1;
                logging("Argument -w detected");
            }
			
            if ( $arg =~ m{(.*n.*)} ) {
                $argNOreport = 1;
                logging("Argument -n detected");
            }
            $arg = 1;
        }

        if ( $arg eq '-i' ) {
            $arg = shift(@ARGV);
            logging("Argument -i detected");

            if ( $arg =~ m{^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$} ) {
                $argIP = $arg;
                logging("IP adress for argument -i is in correct form '$arg'");
            }
            else {
			logging("Wrong IP address format '$arg'\n");logend();
                die("Wrong IP address format '$arg'\n");
            }
            $arg = 1;
        }

        if ( $arg eq '-u' ) {
            $arg = shift(@ARGV);
            logging("Argument -u detected");
                $argURL = $arg;
                logging("URL address in form '$arg'");
            $arg = 1;
        }

        if ( $arg eq '-f' ) {
            $arg = shift(@ARGV);
            logging("Argument -f detected");

            if ( -e "$log_folder$arg" ) {
                $argFILE = $arg;
                logging("file exist with defined path '$arg'");
            }
            else {
				 logging("File not exist or path is wrong for file  '$log_folder$arg'\n");logend();
                die("File not exist or path is wrong for file  '$log_folder$arg'\n");
            }
            $arg = 1;
        }
        if ( $arg eq '-d' ) {
            $arg = shift(@ARGV);
            logging("Argument -d detected");

            if (($arg>0)and($arg<100)) {
                $argLASTdays = $arg;
                logging("Setting time period for last '$arg' days");
				
					
					$argTIMEhigh=GETtimestamp(time());
					
					logging("Seting now '$argTIMEhigh' time as end time");
					
					my $lower_time  = GETepoch($argTIMEhigh)-$arg*24*60*60;
					
					$argTIMElow=GETtimestamp($lower_time);
					
					logging("Seting time before '$arg' days to '$argTIMElow' time as start time");
				
            }
            else {
				 logging("Number of days not in correct format or out of bounds 0 - 100 : '$arg'\n");logend();
                die("Number of days not in correct format or out of bounds 0 - 100 : '$arg'\n");
            }
            $arg = 1;
        }


        if ( $arg eq $wrong_arg ) {
		 logging("Unknown argument '$arg' ");logend();
            die "Unknown argument '$arg' \n";
        }

    }

}

if (    ($argEMAIL or $argTIMElow or $argTIMEhigh  or $argFILE) and (  $argWEB or $argADDRESS or $argNOreport or $argIP or $argURL ) )
{
$argDEFAULT = 0;
}
elsif ($argEMAIL or $argTIMElow or $argTIMEhigh  or $argFILE)
{
$argDEFAULT = 1;
}



logging(
    "	Result of argument processing
		\$argEMAIL '$argEMAIL'
		\$argWEB '$argWEB'
		\$argADDRESS '$argADDRESS' 
		\$argNOreport '$argNOreport' 
		\$argIP '$argIP'
		\$argURL '$argURL'
		\$argFILE '$argFILE'
		\$argDEFAULT '$argDEFAULT'
		\$argTIMElow '$argTIMElow'
		\$argTIMEhigh '$argTIMEhigh'
		\$argLASTdays '$argLASTdays'
"
);

# -----------------------------------------------------------------
############## END OF ARGUMENT PROCESSING





############## RUN OF PROGRAM
####+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

my @total;
my %uniqIP;
my @sortedIP;
my @oneIPtotal;
my %fullDHCP;
my %fullIPdatabase;
my %fullMAC;

if ( $argTIMElow or $argTIMEhigh ) 
{
    logging(
        "Getting all log records between times $argTIMElow and $argTIMEhigh");
    my $epochTIMElow  = GETepoch($argTIMElow);
	$argTIMElow=GETtimestamp($epochTIMElow);
    my $epochTIMEhigh = GETepoch($argTIMEhigh);
	$argTIMEhigh=GETtimestamp($epochTIMEhigh);
	logging("Selecting records between times $epochTIMElow and $epochTIMEhigh");
    GETtotalTIMEbounds( \@total, \%uniqIP, $log_folder, $epochTIMElow,
        $epochTIMEhigh );
}
else 
{
    logging("Getting default log records set from default file");
    GETtotalDEFAULT( $log_folder, \@total, \%uniqIP,$argFILE );
}


logging( "Total number of records for parsing :" . scalar(@total) );

GETuniqIP( \@sortedIP, \%uniqIP );
logging( "Total number of unique IP addresses:" . scalar(@sortedIP) );

GETfullDHCP( $leases_path, \%fullDHCP );
GETfullMAC($leases_path,\%fullMAC);
GETfullIPdatabase( $database_path, \%fullIPdatabase,\%fullMAC);


# print Dumper %fullMAC;

if ( $argDEFAULT ) 
{
    logging("Creating full report");
    HTMLreportFULL( $reports_folder, \@total, \@sortedIP, \%fullDHCP, \%fullIPdatabase, $argTIMElow, $argTIMEhigh);
}
elsif ( ( $argWEB or $argADDRESS ) and !$argNOreport )
 {
    if ($argWEB) {
        logging("Creating briefly HTML report for domains");
        DOMAINglobREPORT( $reports_folder, \@total, \%uniqIP );
    }
    if ($argADDRESS) {
        logging("Creating briefly HTML report for IP addresses");
        IPglobREPORT( $reports_folder, \@total, \%uniqIP,\%fullDHCP,\%fullIPdatabase);
    }

}
elsif ( ( $argWEB or $argADDRESS ) and $argNOreport )
 {
    if ($argWEB) {
        logging(
            "Creating briefly report for domains with command line output ");
        DOMAINglobNOreport( \@total );
    }
    if ($argADDRESS) {
        logging(
            "Creating briefly report for IP addresses with command line output "
        );
        IPglobNOreport( \@total, \%uniqIP,\%fullDHCP,\%fullIPdatabase );
    }
}
elsif ($argIP) 
{
    logging("Creating HTML report for one IP '$argIP' ");

    my %oneIPdomain;
    my @oneIPtime;
    my %oneIPsize;

    GEToneIPtotal( $argIP, \@total, \@oneIPtotal );
        if ( scalar(@oneIPtotal ) ==0 ) {
		 logging("IP '$argIP' is not present in log records ");logend();
            die "IP '$argIP' is not present in log records \n";
        }

    GEToneIPdomain( $argIP, \@oneIPtotal, \%oneIPdomain );
    %oneIPsize = %oneIPdomain;
    GEToneIPsize( $argIP, \@oneIPtotal, \%oneIPsize );
    GEToneIPtime( $argIP, \@oneIPtotal, \@oneIPtime );
    HTMLbodyONE(
        \@total, $reports_folder, $argIP,     \%oneIPdomain, \%oneIPsize,
        \@oneIPtime,     \%fullDHCP, \%fullIPdatabase, $argTIMElow, $argTIMEhigh
    );

}
elsif ($argURL) {
    URLreport( $reports_folder, $argURL, \@total, \%uniqIP,\%fullDHCP,\%fullIPdatabase );
}


my $stop_time = time();
my $run_time  = $stop_time - $start_time;

logging("Aplication ran $run_time seconds or ". ( $run_time / 60 ). " minutes ");
logging("Aplication ending normally");
logging("----------------END OF APPLICATION--------------");


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#################### END OF PROGRAM RUN






################## SUBROUTINES DEFINITION
#====================================================================

######## Subroutine for obtain default log records set from default log file

sub GETtotalDEFAULT {

    # GETtotal ($log_folder,\@total, \%uniqIP,$argFILE)

    my $sublog_folder = shift;
    my $subtotal      = shift;
    my $subuniq       = shift;
	my $default_log= shift;
	
	
	if($default_log eq '0')
	{
    $default_log   = 'access.log.1';
	}
	
logging("Using default log file '$default_log' ");

    open( my $logfile, "<", "$sublog_folder$default_log" ) or die ("Could not open file '$sublog_folder$default_log' $!\n");


    my $increment = 0;

    while (<$logfile>) {

        my @array = split( /\s+/, $_ );

        if ( $array[6] !~ m{(^NONE.*)} ) {

            ${$subtotal}[$increment] = "$array[0] $array[2] $array[4] $array[6]";
            ${$subuniq}{ $array[2] } = 0;
            $increment++;
        }
    }
    close($logfile) or 
	die "Could not close file '$sublog_folder$default_log' \n";


}
#####################
#####################

### Subroutine for obtain advanced log records set after defining time bounds

sub GETtotalTIMEbounds {

# sub GETtotalTIMEbounds(\@total,\%uniqIP,$log_folder,$epochTIMElow,$epochTIMEhigh);

    my $subtotal         = shift;
    my $subuniq          = shift;
    my $sublog_folder    = shift;
    my $subepochTIMElow  = shift;
    my $subepochTIMEhigh = shift;
    my $LOGDIR;
    my $pointer;
    my %FILEtimeLOW;
    my %FILEtimeHIGH;
    my %finalFILES;
    my $lastLINE;
    my @sortedFILES;
    my $increment = 0;
	my $file_sum=0;

    opendir( $LOGDIR, $log_folder );

    while ( $pointer = readdir $LOGDIR ) {
	
		
		
        if ( $pointer =~ m{^access} ) {
			$file_sum++;
            open( FH, "<", "$sublog_folder$pointer" )
              or
			  die "cannot open '$sublog_folder$pointer' $!\n";


            my $field = ( scalar <FH> );

            my @array = split( /\s+/, $field );
            $FILEtimeLOW{$pointer} = $array[0];

            while (<FH>) {
                $lastLINE = $_;
            }

            @array = split( /\s+/, $lastLINE );
            $FILEtimeHIGH{$pointer} = $array[0];

            close FH, or die "cannot close '$sublog_folder$pointer' $!\n";

        }

    }
	
    foreach my $filename ( keys(%FILEtimeLOW) ) {

        if (
            (
                   ( $subepochTIMElow < $FILEtimeLOW{$filename} )
                or ( $subepochTIMEhigh < $FILEtimeLOW{$filename} )
            )
            and (  ( $subepochTIMEhigh > $FILEtimeHIGH{$filename} )
                or ( $subepochTIMElow > $FILEtimeHIGH{$filename} ) )
          )
        {
            $finalFILES{$filename} = $FILEtimeLOW{$filename};

        }

    }
	
	if($file_sum == (keys(%finalFILES)) )
	{
					
	my $timestamp=GETtimestamp( $subepochTIMElow);
	
	print "\nWarning: Using all log files fo processing, time '$timestamp' maybe too old \n";
	logging("Warning: Using all log files fo processing, time '$timestamp' maybe too old ");
	}

    foreach my $filename ( sort { $finalFILES{$b} <=> $finalFILES{$a} }
        keys(%finalFILES) )
    {
        $sortedFILES[$increment] = $filename;
        $increment++;
    }

    @sortedFILES = reverse(@sortedFILES);
	
  my $numero = 0;
  
    foreach my $filename (@sortedFILES) {

        open( my $logfile, "<", "$sublog_folder$filename" ) or die "cannot open '$sublog_folder$filename' $!\n";

		logging("Using file $sublog_folder$filename");
      

        while (<$logfile>) {

            my @array = split( /\s+/, $_ );

            if ( ($array[6] !~ m{^NONE.*}) and(($array[0]>$subepochTIMElow)and($array[0]<$subepochTIMEhigh))) {

                ${$subtotal}[$numero] =
                  "$array[0] $array[2] $array[4] $array[6]";
                ${$subuniq}{ $array[2] } = 0;
                $numero++;
            }
        }
        close($logfile) or die "cannot close '$sublog_folder$filename' $!\n";

    }


}
#####################
#####################

####### Subroutine for create timestamp from desired epoch time

sub GETtimestamp{

###### $timestamp=GETtimestamp($epoch)

my $subepoch=shift;

		my ( $sec, $min, $hour, $mday, $mon, $year ) =
		localtime($subepoch);
		my $subtimestamp =
		( $year + 1900 ) . "-"
		. sprintf( "%02d", $mon + 1 ) . "-"
		. sprintf( "%02d", $mday ) . "T"
		. sprintf( "%02d", $hour ) . ":"
		. sprintf( "%02d", $min ) . ":"
		. sprintf( "%02d", $sec );

		return($subtimestamp);

}


#####################
#####################

####### Subroutine for transfer from argument time to epoch

sub GETepoch {

    ########## my($epoch)=GETepoch($argTIME);

    my $subargTIME = shift;

    my ( $sec, $min, $hour, $mday, $mon, $year );

    if ( $subargTIME =~ m{.*T.*} ) {
        my @array = split( /T/, $subargTIME );

        if ( $array[0] ) {

            my @datearray = split( /-/, $array[0] );

            $year = $datearray[0];
            $mon  = $datearray[1] - 1;
            $mday = $datearray[2];

        }

        if ( $array[1] ) {

            my @timearray = split( /:/, $array[1] );

            if ( $timearray[0] ) {
                $hour = $timearray[0];
            }
            else {
                $hour = 0;
            }

            if ( $timearray[1] ) {
                $min = $timearray[1];
            }
            else {
                $min = 0;
            }

            if ( $timearray[2] ) {
                $sec = $timearray[2];
            }
            else {
                $sec = 0;
            }
        }

        else {
            $sec  = 0;
            $min  = 0;
            $hour = 0;
        }

    }
    else {

        my @datearray = split( /-/, $subargTIME );

        $year = $datearray[0];
        $mon  = $datearray[1] - 1;
        $mday = $datearray[2];

        $sec  = 0;
        $min  = 0;
        $hour = 0;

    }
	
	if(($year<2000)or($year>8000))
	{ 
	 logging("Wrong year size '$year' ");logend();
	die("Wrong year size '$year' \n"); }
	elsif(($mon<0)or($mon>11))
	{ 
	 logging("Wrong month size '". ($mon+1). "'");logend();
	die("Wrong month size '". ($mon+1). "'\n"); }
	elsif(($mday<0)or($mday>31))
	{ 
	 logging("Wrong day size '$mday'");logend();
	die("Wrong day size '$mday'\n"); }
	elsif(($sec<0)or($sec>59))
	{ 
	 logging("Wrong seconds size '$sec'");logend();
	die("Wrong seconds size '$sec'\n"); }
	elsif(($min<0)or($min>59))
	{ 
	 logging("Wrong minutes size '$min'");logend();
	die("Wrong minutes size '$min'\n"); }
	elsif(($hour<0)or($hour>23))
	{ 
	 logging("Wrong seconds size '$hour'");logend();
	die("Wrong seconds size '$hour'\n"); }
	
	
    my $epoch = timelocal( $sec, $min, $hour, $mday, $mon, $year );
    return ($epoch);

}

#####################
#####################

### ########Subroutine for creation of sorted array of uniq IP adresses

sub GETuniqIP {

    ########## GET sorted(\@sortedIP, \@uniqIP)

    my $subsortedIP = shift;
    my $subuniqIP   = shift;

    my @sortip;

    my $increment = 0;
    foreach my $ip ( keys( %{$subuniqIP} ) ) {

        $sortip[$increment] = $ip;
        $increment++;
    }

    @{$subsortedIP} = map { $_->[0] }
      sort { $a->[1] cmp $b->[1] }
      map { [ $_, sprintf( "%03.f%03.f%03.f%03.f", split( /\./, $_ ) ) ] }
      @sortip;

}
#####################
#####################

### Subroutine which obtain all log records which  belong to one IP from general log records set

sub GEToneIPtotal {

    ########## GEToneIPtotal(IP,\@total, \@oneIPtotal)

    my $oneIP         = shift;
    my $subtotal      = shift;
    my $suboneIPtotal = shift;

    my $increment = 0;

    foreach my $row ( @{$subtotal} ) {

        my @array = split( /\s+/, $row );

        if ( $array[1] eq $oneIP ) {

            ${$suboneIPtotal}[$increment] = $row;
            $increment++;
        }
    }

}
#####################
#####################

### Subroutine which obtaion hash of uniq domains for defined IP with sum of occurs

sub GEToneIPdomain {

    #GEToneIPdomain(IP,\@oneIPtotal,\%oneIPdomain )

    my $subIP          = shift;
    my $suboneIPtotal  = shift;
    my $suboneIPdomain = shift;

### First part, receive hash of uniq domains

    foreach my $row ( @{$suboneIPtotal} ) {

        my @array = split( /\s+/, $row );

        if ( $array[3] =~ m{(^.*?\.\w*\/)} ) {

            ${$suboneIPdomain}{$1} = 0;
        }

    }
### Second part, fill hash with sum of uniq domain occurs

    foreach my $domain ( keys( %{$suboneIPdomain} ) ) {

        foreach my $row ( @{$suboneIPtotal} ) {

            my @array = split( /\s+/, $row );

            if ( $array[3] =~ m{(^$domain)} ) {

                ${$suboneIPdomain}{$1} = ${$suboneIPdomain}{$1} + 1;

            }

        }

    }

}
#####################
#####################

### Subroutine which obtain field with time and URL from one IP log record 

sub GEToneIPtime {
## GEToneIPtimeALL(IP,\@oneIPtotal,\@oneIPtime)

    my $subIP         = shift;
    my $suboneIPtotal = shift;
    my $suboneIPtime  = shift;
    my $increment         = 0;

    foreach my $row ( @{$suboneIPtotal} ) {

        my @array = split( /\s+/, $row );

        ${$suboneIPtime}[$increment] = "$array[0] $array[3]";
        $increment++;
    }

}
#####################
#####################

######### Subroutine which obtain hash of uniq domains of one IP and transfered size for each domain

sub GEToneIPsize {
## GEToneIPsize(IP,\@oneIPtotal,\%oneIPsize)

    my $subIP         = shift;
    my $suboneIPtotal = shift;
    my $suboneIPsize  = shift;

    #### Empty hash received in reference
    foreach my $row ( keys( %{$suboneIPsize} ) ) {
        ${$suboneIPsize}{$row} = 0;
    }

    foreach my $domain ( keys( %{$suboneIPsize} ) ) {

        foreach my $row ( @{$suboneIPtotal} ) {

            my @array = split( /\s+/, $row );

            if ( $array[3] =~ m{(^$domain)} ) {

                ${$suboneIPsize}{$1} = ${$suboneIPsize}{$1} + $array[2];

            }

        }

    }
}
#####################
#####################

######### Subroutine which obtain hash of IP and identification data from DHCPD file

sub GETfullDHCP {

    ########## GETfullDHCP($leases_path,\%fullDHCP)

    my $subleases_path = shift;
    my $subfullDHCP    = shift;

    open( my $dhcpd_FILE, $subleases_path ) or die "Could not open lease file\n";


    undef %{$subfullDHCP};
    my $pending;
    my @temp;
    my $client;
    my $IP;

    while (<$dhcpd_FILE>) {

        if ( $_ !~ m{^#.*|^\s*$} ) {

            $_ =~ s{^\s+}{};
            $_ =~ s{\s+$}{};

            push @temp, $_;

            if ( @temp > 12 ) {
                shift @temp;
            }

            if ( $_ =~ m{^\}.*} ) {
                $pending = '';
                $client  = 0;

                while ( $temp[0] !~ m{^lease.*} ) {
                    shift @temp;
                }

                foreach my $var (@temp) {

                    if ( $var =~ m{^lease.*} ) {
                        my @array = split( /\s+/, $var );
                        $IP = $array[1];
                    }

                    if ( $var =~ m{^hardware.*} ) {
                        my @array = split( /\s+/, $var );
                        $pending = substr( $array[2], 0, 17 );
                    }

                    if ( $var =~ m{^client.*} ) {
                        my @array = split( /\s+/, $var );
                        $pending = $pending . " " . substr( $array[1], 1, -2 );
                        $client = 1;
                    }
                }

                if ( $client == 0 ) {
                    $pending = $pending . " N/A";
                }

                ${$subfullDHCP}{$IP} = $pending;
            }

        }
    }

    close($dhcpd_FILE) or die "Could not close leases file\n";


}
#####################
#####################

######### Subroutine which obtain hash of MAC and identification data from DHCPD file

sub GETfullMAC {

    ########## GETfullDHCP($leases_path,\%fullDHCP)

    my $subleases_path = shift;
    my $subfullMAC    = shift;

    open( my $dhcpd_FILE, $subleases_path ) or die "Could not open lease file\n";


    undef %{$subfullMAC};
    my $pending;
    my @temp;
    my $client;
    my $MAC;

    while (<$dhcpd_FILE>) {

        if ( $_ !~ m{^#.*|^\s*$} ) {

            $_ =~ s{^\s+}{};
            $_ =~ s{\s+$}{};

            push @temp, $_;

            if ( @temp > 12 ) {
                shift @temp;
            }

            if ( $_ =~ m{^\}.*} ) {
                $pending = '';
                $client  = 0;

                while ( $temp[0] !~ m{^lease.*} ) {
                    shift @temp;
                }

                foreach my $var (@temp) {

                    if ( $var =~ m{^lease.*} ) {
                        my @array = split( /\s+/, $var );
                        $pending = $pending.$array[1];
                    }

                    if ( $var =~ m{^hardware.*} ) {
                        my @array = split( /\s+/, $var );
                        $MAC = substr( $array[2], 0, 17 );
                    }

                    if ( $var =~ m{^client.*} ) {
                        my @array = split( /\s+/, $var );
                        $pending = $pending . " " . substr( $array[1], 1, -2 );
                        $client = 1;
                    }
                }

                if ( $client == 0 ) {
                    $pending = $pending . " N/A";
                }

                ${$subfullMAC}{$MAC} = $pending;
            }

        }
    }

    close($dhcpd_FILE) or die "Could not close leases file\n";


}

#####################
#####################

#### Subroutine which obtaion hash of IP and other identification from database file with emails


sub GETfullIPdatabase {

    ###### GETfullIPdatabase($database_path,\%fullIPdatabase,\%fullMAC)

    my $subdatabase_path  = shift;
    my $subfullIPdatabase = shift;
	my $subfullMAC=shift;

    open( my $database_FILE, $subdatabase_path ) or die "Could not open database file\n";


    my $IP;

    undef %{$subfullIPdatabase};

    while (<$database_FILE>) {

        if ( $_ !~ m{^#.*|^\s*$} ) {

            $_ =~ s{^\s+}{};
            $_ =~ s{\s+$}{};

            my @array = split( /\s+/, $_ );

            if ( $_ =~ m{^\d} ) {
                $IP = $array[0];
                shift @array;
				
				if ($IP=~m{(?:[a-f0-9]{2}\:){5}[a-f0-9]{2}})
				{
					if(${$subfullMAC}{$IP} )
						{
							my @MACarrray=split(/\s+/, ${$subfullMAC}{$IP} );
							my $MACIP=$MACarrray[0];
							shift @MACarrray;
							${$subfullIPdatabase}{$MACIP}="@array";
						}
					else
						{
							logging("MAC address '$IP' couldn't found in leases, skipping ");
						}
				}
				else
				{
                ${$subfullIPdatabase}{$IP} = "@array";
				}
            }

        }
    }
    close($database_FILE) or die "Could not close database file\n";


}
#####################
#####################

#### Subroutine which send email through extern email account using credentials via SSL connection

sub SendEmailAttachment {

    ####### 	SendEmailAttachment($reportPath,$fileName,$subIP,$Name,$Email,$argTIMElow, $argTIMEhigh );

    my $subFilePath  = shift;
    my $subFileName  = shift;
	my $subIP = shift;
	my $subName =shift;
    my $to           = shift;
	my $subargTIMElow =shift;
	my $subargTIMEhigh = shift;


	
    my $bound  = "MMMMMMMMMMMMM";
	
	### Put your email credentials and other informations here
    my $server = 'smtp@somedomain.com';
    my $user   = 'user@somedomain.com';
    my $pass   = 'password';
    my $from   = 'user@somedomain.com';
	
	
    my @attachment;
	
	
     my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
    $atime,$mtime,$ctime,$blksize,$blocks)
    = stat($subFilePath);
	
	if ($size<(10*1024*1024))
	{
	logging("Sending email");
	
    open( DATA, $subFilePath ) or die ("Could not open file '$subFilePath' $!\n");

    while (<DATA>) {
        push( @attachment, $_ );
    }
    close(DATA);

    my $smtp = Net::SMTP::SSL->new( $server, Port => 465) or die ("Could not connect to server $!\n");

    $smtp->auth( $user, $pass ) or die ("Authentication failed! $!\n");

    $smtp->mail("$user");
    $smtp->to($to);
    $smtp->data();
    $smtp->datasend( "From: " . $user . "\n" );

    ###########  $smtp->datasend("Reply-To: <noreply\@gmail.com>\n");
    $smtp->datasend( "To: \<" . $to . "\>\n" );
    $smtp->datasend("Subject: Web activity report $subName \($subIP \) $subargTIMElow - $subargTIMEhigh \n");
    $smtp->datasend("MIME-Version: 1.0\n");
    $smtp->datasend("Content-Type: multipart/mixed;\n\t boundary=\"$bound\"\n");
    $smtp->datasend("\n--$bound\n");
    $smtp->datasend("Content-Type: text/plain\n\n");
    $smtp->datasend("Summary of web activity report for subject $subName \($subIP\)\n");
	$smtp->datasend("Reported time period $subargTIMElow - $subargTIMEhigh\n");
    $smtp->datasend("See attached file for more details\n\n");
    $smtp->datasend("\n--$bound\n");
    $smtp->datasend("Content-Disposition: attachment; file= \"$subFilePath\"\n");
    $smtp->datasend("Content-Type: application/html; name= \"$subFileName\"\n\n");
    $smtp->datasend("@attachment\n\n");
    $smtp->datasend("\n--$bound\n");
    $smtp->datasend();
    $smtp->dataend();
    $smtp->send();
    $smtp->quit();
	
	logging("Email successfully send");
	}
	else
	{
	 my( $sizeConv, $unit ) = sizecon( $size );
	logging("WARNING: File '$subFilePath' couldn't be send as attachment, file size is greater than 10 MB, file size: '$sizeConv $unit', email not send");
	}
}

#####################
#####################

#### Subroutine which send email through extern email account using credentials via LOGIN or another lower login method

# sub SendEmailAttachment {

    ########## SendEmailAttachment($FilePath,$fFileName,$Email,$TimeRange);
	# use Net::SMTP_auth;
	
    # my $subFilePath  = shift;
    # my $subFileName  = shift;
    # my $to           = shift;
    # my $subTimeRange = shift;

    # my $bound  = "MMMMMMMMMMMMM";
    # my $server = 'smtp.googlemail.com';
    ######## my $user   = 'parser.report.sender@gmail.com';
	# my $user   = 'parser.report.sender';
    # my $pass   = 'posielaniereportov';
    # my $from   = 'parser.report.sender@gmail.com';
    # my @attachment;
	
	
     # my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
    # $atime,$mtime,$ctime,$blksize,$blocks)
    # = stat($subFilePath);
	
	# if ($size<(10*1024*1024))
	# {
	
	
	

    # open( DATA, $subFilePath ) || die("Could not open the file");

    # while (<DATA>) {
        # push( @attachment, $_ );
    # }
    # close(DATA);
	
	# $smtp = Net::SMTP_auth->new('mail.chemnet.sk')or die("Could not connect to server $!\n");
	###### Other login methods than 'LOGIN' can be obtain from server via command
	###### print  $smtp->auth_types ();
    # $smtp->auth('LOGIN', $user,$pass)or die "Authentication failed! $!\n";
	
    # $smtp->mail('parser.report.sender@gmail.com');
    # $smtp->to($to);
    # $smtp->data();
    # $smtp->datasend( "From: " . $user . "\n" );
    ####### $smtp->datasend("Reply-To: <noreply\@gmail.com>\n");
    # $smtp->datasend( "To: \<" . $to . "\>\n" );
    # $smtp->datasend("Subject: Web activity report $subTimeRange \n");
    # $smtp->datasend("MIME-Version: 1.0\n");
    # $smtp->datasend("Content-Type: multipart/mixed;\n\t boundary=\"$bound\"\n");
    # $smtp->datasend("\n--$bound\n");
    # $smtp->datasend("Content-Type: text/plain\n\n");
    # $smtp->datasend(
        # "Summary of web activity report for time period $subTimeRange\n");
    # $smtp->datasend("See attached file for more details\n\n");
    # $smtp->datasend("\n--$bound\n");
    # $smtp->datasend(
        # "Content-Disposition: attachment; file= \"$subFilePath\"\n");
    # $smtp->datasend(
        # "Content-Type: application/html; name= \"$subFileName\"\n\n");
    # $smtp->datasend("@attachment\n\n");
    # $smtp->datasend("\n--$bound\n");
    # $smtp->datasend();
    # $smtp->dataend();
    # $smtp->send();
    # $smtp->quit();
	
	
	# }
	# else
	# {
	 # my( $sizeConv, $unit ) = sizecon( $size );
	# logging("WARNING: File '$subFilePath' couldn't be send as attachment, file size is greater than 10 MB, file size: '$sizeConv $unit', email not send");
	# }
# }



#####################
#####################

##### Subroutine for main creation of HTML body used for full reports

sub HTMLbody {

# sub HTMLbody (\@total,$report_folder,IP,\%oneIPdomain,\%oneIPsize, \@oneIPtime,\%fullDHCP,\%fullIPdatabase, $argTIMElow, $argTIMEhigh )

	my $subtotal = shift;
    my $subreport_folder  = shift;
    my $subIP             = shift;
    my $suboneIPdomain    = shift;
    my $suboneIPsize      = shift;
    my $suboneIPtime      = shift;
    my $subfullDHCP       = shift;
    my $subfullIPdatabase = shift;
	my $subargTIMElow =shift;
	my $subargTIMEhigh = shift;
	
	
	if (!($subargTIMElow and $subargTIMEhigh ))
	{
	my $TimeRange=GetTimeRange($subtotal);
	my @timearray=split(/\s+/,$TimeRange);
	
	$subargTIMElow=$timearray[0];
	$subargTIMEhigh=$timearray[2];

	}
	
    my $printfile;
	my $allowed;
	my $fileName="$subIP"."_report.html";
	
	if(${$subfullIPdatabase}{$subIP} )
	{
	my @array = split( /\s+/,${$subfullIPdatabase}{$subIP} );
	$allowed=shift @array;
	}
	else
	{
	$allowed='E';
	}
	
	if ($allowed ne 'D')
	{
	
	my $reportPath="$subreport_folder$subIP" . ".report.html";
	
    open FH, ">$reportPath" or die "Couldn't open html report file for IP $subIP\n";


    print FH
"\<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\"\>
\<html xmlns=\"http://www.w3.org/1999/xhtml\"\>
	\<head\>
		\<meta name=\"Proxy_report\" content=\"HTML report for $subIP containing various data informations\" /\>
		\<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" /\>
		\<title\>Report for $subIP \</title\>

	\</head\>
	\<body\>";


    $printfile =
      HTMLheader( $subIP, $subfullDHCP, $subfullIPdatabase, $suboneIPsize,
        $suboneIPtime, $suboneIPdomain );
    print FH "$printfile";

    $printfile = HTMLtableUNIQipURLsum($suboneIPdomain);
    print FH "$printfile";

    $printfile = HTMLtableUNIQipURLsize($suboneIPsize);
    print FH "$printfile";
	
    $printfile = HTMLtableUNIQipURLall($suboneIPtime);
    print FH "$printfile";


    print FH "	\</body\>
\</html\>";

    close FH;
	
	
if ($argEMAIL)
{
	if ( ${$subfullIPdatabase}{$subIP} ) {

        my @array = split( /\s+/, ${$subfullIPdatabase}{$subIP} );
		shift @array;
		my $Email=shift @array;
		my $Name="@array";

		SendEmailAttachment($reportPath,$fileName,$subIP,$Name,$Email,$subargTIMElow, $subargTIMEhigh );
    }

}


}
	
	
}

#####################
#####################

##### Subroutine for main creation of HTML body used for one IP (Almost same as HTMLbody)

sub HTMLbodyONE {

# sub HTMLbody (\$total, $report_folder,IP,\%oneIPdomain,\%oneIPsize, \@oneIPtime,\%fullDHCP,\%fullIPdatabase, $argTIMElow, $argTIMEhigh )

	my $subtotal = shift;
    my $subreport_folder  = shift;
    my $subIP             = shift;
    my $suboneIPdomain    = shift;
    my $suboneIPsize      = shift;
    my $suboneIPtime      = shift;
    my $subfullDHCP       = shift;
    my $subfullIPdatabase = shift;
	my $subargTIMElow =shift;
	my $subargTIMEhigh = shift;
    my $printfile;
	my $allowed;
	
	my $fileName="$subIP"."_report.html";
	
	if (!($subargTIMElow and $subargTIMEhigh ))
	{
	my $TimeRange=GetTimeRange($subtotal);
	my @timearray=split(/\s+/,$TimeRange);
	
	$subargTIMElow=$timearray[0];
	$subargTIMEhigh=$timearray[2];

	}

	
	if(${$subfullIPdatabase}{$subIP} )
	{
	my @array = split( /\s+/,${$subfullIPdatabase}{$subIP} );
		$allowed=shift @array;
	}
	else
	{
	$allowed='E';
	}
	
	if ($allowed ne 'D')
	{
	
                my ( $sec, $min, $hour, $mday, $mon, $year ) =
                  localtime( time() );
                my $reportPath ="$subreport_folder$subIP"."_".
                    ( $year + 1900 ) . "-"
                  . sprintf( "%02d", $mon + 1 ) . "-"
                  . sprintf( "%02d", $mday ) . "T"
                  . sprintf( "%02d", $hour )
                  . sprintf( "%02d", $min ) 
                  . sprintf( "%02d", $sec ).".report.html";	

    open FH, ">$reportPath" or die "Couldn't open html report file for IP $subIP\n";
	  
    print FH
"\<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\"\>
\<html xmlns=\"http://www.w3.org/1999/xhtml\"\>
	\<head\>
		\<meta name=\"Proxy_report\" content=\"HTML report for $subIP containing various data informations\" /\>
		\<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" /\>
		\<title\>Report for $subIP \</title\>

	\</head\>
	\<body\>";

    $printfile =
      HTMLheader( $subIP, $subfullDHCP, $subfullIPdatabase, $suboneIPsize,
        $suboneIPtime, $suboneIPdomain );
    print FH "$printfile";

    $printfile = HTMLtableUNIQipURLsum($suboneIPdomain);
    print FH "$printfile";

    $printfile = HTMLtableUNIQipURLsize($suboneIPsize);
    print FH "$printfile";

    $printfile = HTMLtableUNIQipURLall($suboneIPtime);
    print FH "$printfile";

    print FH "	\</body\>
\</html\>";

    close FH;
	
	
if ($argEMAIL)
{
	if ( ${$subfullIPdatabase}{$subIP} ) {

        my @array = split( /\s+/, ${$subfullIPdatabase}{$subIP} );
		shift @array;
		my $Email=shift @array;
		my $Name="@array";

		SendEmailAttachment($reportPath,$fileName,$subIP,$Name,$Email,$subargTIMElow, $subargTIMEhigh );
    }

}


}

}
#####################
#####################

#### Subroutine for getting $TimeRange

sub GetTimeRange {
	# my $TimeRange=GetTimeRange(\$suboneIPtime);


my $suboneIPtime=shift;
my $finalstring;


my @array = split( /\s+/,${$suboneIPtime}[0] );

				  my $time1=GETtimestamp($array[0]);

 @array = split( /\s+/,${$suboneIPtime}[scalar(@{$suboneIPtime})-1] );

				  my $time2=GETtimestamp($array[0]);

$finalstring="$time1 - $time2";

return($finalstring);

}

sub GetTimeRangeT {
	# my $TimeRange=GetTimeRange(\$suboneIPtime);


my $suboneIPtime=shift;
my $finalstring;


my @array = split( /\s+/,${$suboneIPtime}[0] );

				  my $time1=GETtimestamp($array[0]);

 @array = split( /\s+/,${$suboneIPtime}[scalar(@{$suboneIPtime})-1] );

				  my $time2=GETtimestamp($array[0]);

$finalstring="$time1 - $time2";

return($finalstring);

}



#####################
#####################

#### Subroutine for convert junk of size in bytes to four digit size with corresponding unit

sub sizecon {

 # ( $size, $unit ) = sizecon( $numeric );
 
    my $size = shift;
    my $unit = 'B';

    if ( $size >= 1024 ) {
        $size = $size / 1024;
        $unit = 'KB';

        if ( $size >= 1024 ) {
            $size = $size / 1024;
            $unit = 'MB';

            if ( $size >= 1024 ) {
                $size = $size / 1024;
                $unit = 'GB';
            }
        }
    }
    if ( $unit ne 'B' ) {
        $size = sprintf "%.4f", $size;
    }
    return ( $size, $unit );
}
#####################
#####################

##### Subroutine which print table of unique domain and sum of occurs to HTML report file for one IP

sub HTMLtableUNIQipURLsum {
##### HTMLtableUNIQipURLsum ($suboneIPdomain)

    my $array = shift;
    my $result;

    $result = "
		\<h2\> Uniq domain sum occurs \</h2\>
		\<table border=\"1\"\>
			\<tr\>
				\<th\>URL\</th\>
				\<th\>POCET\</th\>
			\</tr\>";

    foreach my $value ( sort { ${$array}{$b} <=> ${$array}{$a} } keys %{$array} ) {

        $result = $result . "
			\<tr\>
				\<td\>$value\</td\>
				\<td\>${$array}{$value}\</td\>
			\</tr\>";

    }

    $result = $result . "
		\</table\>
";

}

#####################
#####################

##### Subroutine which print table of unique domains and size of transfered data with unit transformation to HTML report file for one IP

sub HTMLtableUNIQipURLsize {
##### HTMLtableUNIQipURLsum ($oneIPsize)

    my $array = shift;
    my $result;
    my $size;
    my $unit;

    $result = "
		\<h2\> Amount of transfered data for uniq domains \</h2\>
		\<table border=\"1\"\>
			\<tr\>
				\<th\>URL\</th\>
				\<th\>VELKOST\</th\>
			\</tr\>";

    foreach my $value ( sort { ${$array}{$b} <=> ${$array}{$a} } keys %{$array} ) {

        ( $size, $unit ) = sizecon( ${$array}{$value} );

        $result = $result . "
			\<tr\>
				\<td\>$value\</td\>
				\<td\>$size $unit\</td\>
			\</tr\>";

    }

    $result = $result . "
		\</table\>
";

}

#####################
#####################

##### Subroutine which print table of all time log records to HTML report file for one IP

sub HTMLtableUNIQipURLall {
##### HTMLtableUNIQipURLall ($oneIPtime)

    my $subtime = shift;
    my $result;

    $result = "
		\<h2\> All HTTP requests with timestamp \</h2\>
		\<table border=\"1\"\>
			\<tr\>
				\<th\>Cas\</th\>
				\<th\>URL\</th\>
			\</tr\>";

    foreach my $row ( @{$subtime} ) {
        my @array = split( /\s+/, $row );

    my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime( $array[0] );
    my $timestamp =
        ( $year + 1900 ) . "-"
      . sprintf( "%02d", $mon + 1 ) . "-"
      . sprintf( "%02d", $mday ) . "_"
      . sprintf( "%02d", $hour ) . ":"
      . sprintf( "%02d", $min ) . ":"
      . sprintf( "%02d", $sec );
        $result = $result . "
			\<tr\>
				\<td\>" . $timestamp. "\</td\>
				\<td\>$array[1]\</td\>
			\</tr\>";



    }

    $result = $result . "
		\</table\>
";

}
#####################
#####################

##### Subroutine which print table of various indentification information to HTML report file

sub HTMLheader {

# $printfile = HTMLheader($subIP,$subfullDHCP,$subfullIPdatabase,$suboneIPsize,$suboneIPtime,$suboneIPdomain);

    my $subIP             = shift;
    my $subfullDHCP       = shift;
    my $subfullIPdatabase = shift;
    my $suboneIPsize      = shift;
    my $suboneIPtime      = shift;
    my $suboneIPdomain    = shift;
	
    my $result;
    my $sizeSUM   = 0;
    my $allSUM    = 0;

	my $TimeRange=GetTimeRange($suboneIPtime);
	
    $result = "
\<h2\> Identification header fo IP $subIP\</h2\>
	\<table border=\"1\"\>
";

    if ( ${$subfullIPdatabase}{$subIP} ) {

        my @array = split( /\s+/, ${$subfullIPdatabase}{$subIP} );
        shift @array;

        $result = $result . "
		\<tr\>
			\<td\> IP \</td\>
			\<td\> $subIP\</td\>
		\</tr\>";

        $result = $result . "
		\<tr\>
			\<td\> Email \</td\>
			\<td\>$array[0] \</td\>
		\</tr\>";
        shift @array;

        $result = $result . "
		\<tr\>
			\<td\>Name\</td\>
			\<td\> " . "@array" . " \</td\>
		\</tr\>";

    }

    elsif ( ${$subfullDHCP}{$subIP} ) {
        my @array = split( /\s+/, ${$subfullDHCP}{$subIP} );

        $result = $result . "
		\<tr\>
			\<td\> IP \</td\>
			\<td\> $subIP \</td\>
		\</tr\>";

        $result = $result . "
		\<tr\>
			\<td\> MAC \</td\>
			\<td\>$array[0] \</td\>
		\</tr\>";
        shift @array;

        $result = $result . "
		\<tr\>
			\<td\> Hostname \</td\>
			\<td\>$array[0] \</td\>
		\</tr\>";

    }


    $result = $result . "
		\<tr\>
			\<td\> Total unique domains\</td\>
			\<td\>".keys( %{$suboneIPdomain})." \</td\>
		\</tr\>";

    $allSUM   = @{$suboneIPtime};
    $result = $result . "
		\<tr\>
			\<td\> All HTTP request</td\>
			\<td\> $allSUM \</td\>
		\</tr\>";

    foreach my $something ( keys( %{$suboneIPsize} ) ) {
        $sizeSUM = $sizeSUM + ${$suboneIPsize}{$something};
    }
    my ( $size, $unit ) = sizecon($sizeSUM);
    $result = $result . "
		\<tr\>
			\<td\> Transfered data \</td\>
			\<td\>" . $size . " " . $unit . " \</td\>
		\</tr\>";
		
    $result = $result . "
		\<tr\>
			\<td\> Data time period \</td\>
			\<td\>" . $TimeRange . " \</td\>
		\</tr\>";

    my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime( time() );
    my $timestamp =
        ( $year + 1900 ) . "-"
      . sprintf( "%02d", $mon + 1 ) . "-"
      . sprintf( "%02d", $mday ) . " "
      . sprintf( "%02d", $hour ) . ":"
      . sprintf( "%02d", $min ) . ":"
      . sprintf( "%02d", $sec );
		
    $result = $result . "
		\<tr\>
			\<td\> Time of report creation\</td\>
			\<td\> " .$timestamp. " \</td\>
		\</tr\>";

    $result = $result . "
	\</table\>
";

}

#####################
#####################

######### Subroutine for logging events to log file, needed argument is only string of data
sub logging {

    ####### log($text)
    my $subtext = shift;
    my $timestamp;

    my $OWN_LOG;

    if ( -e $own_log_path ) {
        open $OWN_LOG, ">>$own_log_path" or die "Unable to open log file\n";
    }
    else {
        open $OWN_LOG, ">$own_log_path"
          or die "Unable to write to own log file\n";
    }

	  	  $timestamp=GETtimestamp(time());

    print $OWN_LOG time() . " ($timestamp) $subtext \n";

    close $OWN_LOG;

}
#####################
#####################

######### Subroutine for log end of application 
sub logend {

logging("WARNING: Premature end of application");
logging("----------------END OF APPLICATION--------------");

}



#####################
#####################

######### Subroutine for display help in command line output after receiving argument -h

sub basic_help {

    print '
	proxyparser.pl - Utility for parsing squid access logs and generating reports for each IP
	
		-h	Display help 
		-e	Enable sending emails after generate report for one or each IP
		-w	Create global report for web domains (HTML file)
		-a	Create global report for IP adresses (HTML file)
		-n Supress file creation for -a and -w, output is put directly to screen
		-i	Followed by IP adress create HTML report for specified IP 
		-f Specify file for processing, file must be located in log folder (default processed file is access.log.1)
		-u Quick report for specified URL, it can be domain, part of domain, or whole word
		-d Specify last X days from actual time and date as time boundaries for log records 
		-t Specify time boundaries for log records to process, must be specified one or two boundaries, in case of one
				boundary, is second boudary automatically actual time. Time format is YYYY-MM-DDTHH:MM:SS, daily hour 
				part after T is not mandatory, Examples:
					-t 2010-05-13T08:00:00 2010-05-18T20:34:15
					-t 2010-05-13 2010-05-18
					-t 2010-05-14
			
		EXAMPLES:
		
		proxyparser.pl			Default run of program, creating of HTML reports for all IP adresses
		proxyparser.pl -e		Default run of program, after creation of HTML report for each IP is this report send to email
						specified in configuration file database.txt
		proxyparser.pl -w		Create global HTML report with all unique domains and informations about size and sum of visits
		proxyparser.pl -a		Create global HTML report with all unique IP adresses and informations about size, sum of visits
						and unique visits
		proxyparser.pl -n 		Doing nothing, -n switch can be combined only with -a or -w or both together
		proxyparser.pl -naw		Create global IP and domain report to screen
		proxyparser.pl -i 10.10.10.1	Create standart HTML report for desired IP address
		proxyparser.pl -u facebook	Display summary of visits for desired web URL or part of domain
		proxyparser.pl -f access.log.16 Start default run of program but source of log records is desired log file
		proxyparser.pl -t \'2010-05-13 2010-05-18\'	Source of data are log records between time ranges from all log files
		proxyparser.pl -t \'2010-05-13\'	Source of data are log records from defined date to now
';

}
#####################
#####################

### Subroutine which create HTML report for each IP address after default call of program

sub HTMLreportFULL {

 # HTMLreportFULL($report_folder,\@total,\@sortedIP,\%fullDHCP,\%fullIPdatabase, $argTIMElow, $argTIMEhigh)

    my $subreport_folder  = shift;
    my $subtotal          = shift;
    my $subsortedIP       = shift;
    my $subfullDHCP       = shift;
    my $subfullIPdatabase = shift;
	my $subargTIMElow =shift;
	my $subargTIMEhigh = shift;
	

    my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime( time() );

    $subreport_folder =
        $subreport_folder
      . "Full_Report_"
      . ( $year + 1900 ) . "-"
      . sprintf( "%02d", $mon + 1 ) . "-"
      . sprintf( "%02d", $mday );
    $subreport_folder =
        $subreport_folder . "T"
      . sprintf( "%02d", $hour )
      . sprintf( "%02d", $min )
      . sprintf( "%02d", $sec ) . "/";

    mkdir $subreport_folder;

    my %own_oneIPdomain;
    my %own_oneIPsize;
    my @own_oneIPtime;
    my @own_oneIPtotal;

    foreach my $IP ( @{$subsortedIP} ) {

        undef %own_oneIPdomain;
        undef %own_oneIPsize;
        undef @own_oneIPtime;
        undef @own_oneIPtotal;


        logging("Start of information collecting for $IP");
		
        GEToneIPtotal( $IP, $subtotal, \@own_oneIPtotal );
        GEToneIPdomain( $IP, \@own_oneIPtotal, \%own_oneIPdomain );
        %own_oneIPsize = %own_oneIPdomain;
        GEToneIPsize( $IP, \@own_oneIPtotal, \%own_oneIPsize );
        GEToneIPtime( $IP, \@own_oneIPtotal, \@own_oneIPtime );
		
		logging("End of information collecting for $IP");
		
		logging("Start of creating HTML report file for $IP");
		
        HTMLbody(
            $subtotal, $subreport_folder, $IP,             \%own_oneIPdomain,
            \%own_oneIPsize,   \@own_oneIPtime, $subfullDHCP,
            $subfullIPdatabase, $argTIMElow, $argTIMEhigh);

		logging("End of creating HTML report file for $IP");
    }

}

#####################
#####################

#### Subroutine for global IP report without creating HTML file

sub IPglobNOreport {

    # IPglobNOreport(\@total,\%uniqIP,\%fullDHCP,\%fullIPdatabase)

    my $subtotal  = shift;
    my $subuniqIP = shift;
	my $subfullDHCP =shift;
	my $subfullIPdatabase=shift;
	
    my %IPsize   = %{$subuniqIP};
    my %IPdomain = %{$subuniqIP};
    my %IPall    = %{$subuniqIP};
    my %uniqdomains;

    my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime( time() );
    my $timestamp =
        ( $year + 1900 ) . "-"
      . sprintf( "%02d", $mon + 1 ) . "-"
      . sprintf( "%02d", $mday ) . " "
      . sprintf( "%02d", $hour ) . ":"
      . sprintf( "%02d", $min ) . ":"
      . sprintf( "%02d", $sec );
	  
	my $TimeRange=GetTimeRange($subtotal);
	
    foreach ( @{$subtotal} ) {

        my @array = split( /\s+/, $_ );

        $IPsize{ $array[1] } = $IPsize{ $array[1] } + $array[2];
        $IPall{ $array[1] }++;

        if ( $array[3] =~ m{(^.*?\.\w*\/)} ) {
            $uniqdomains{$1} = 0;
        }

    }

    foreach my $IP ( keys(%uniqIP) ) {
        my @oneIPtotal;
        GEToneIPtotal( $IP, $subtotal, \@oneIPtotal );

        my %oneIPdomain;
        GEToneIPdomain( $IP, \@oneIPtotal, \%oneIPdomain );

        $IPdomain{$IP} = keys(%oneIPdomain);

    }	
	print "\n	Global IP report generated at $timestamp with data time period '$TimeRange'\n\n";
    print "\n	IP	|	Size	|	Identification	\n------------------------------------------------------------\n";
	
    foreach my $IP ( sort { $IPsize{$b} <=> $IPsize{$a} } keys(%IPsize) ) {
        my ( $size, $unit ) = sizecon( $IPsize{$IP} );
		
		if ( ${$subfullIPdatabase}{$IP} ) {

        my @array = split( /\s+/, ${$subfullIPdatabase}{$IP} );
		shift @array;
		shift @array;
		print "$IP	|	$size $unit	|	@array	\n";
    }

    elsif ( ${$subfullDHCP}{$IP} ) {
		print "$IP	|	$size $unit	|	${$subfullDHCP}{$IP}	\n";
		}

    }
	
    print "\n\n\n\n	IP	|	Unique domains sum	|	Identification	\n------------------------------------------------------------\n";
    foreach my $IP ( sort { $IPdomain{$b} <=> $IPdomain{$a} } keys(%IPdomain) )
    {
		if ( ${$subfullIPdatabase}{$IP} ) {

        my @array = split( /\s+/, ${$subfullIPdatabase}{$IP} );
		shift @array;
		shift @array;
		print "$IP	|	$IPdomain{$IP}	|	@array\n";
    }

    elsif ( ${$subfullDHCP}{$IP} ) {
		print "$IP	|	$IPdomain{$IP}	|	${$subfullDHCP}{$IP}	\n";
		}
		
    }
    print "\n\n\n\n	IP	|	HTTP requests sum	|	Identification	\n------------------------------------------------------------\n";

    foreach my $IP ( sort { $IPall{$b} <=> $IPall{$a} } keys(%IPall) ) {

		if ( ${$subfullIPdatabase}{$IP} ) {

        my @array = split( /\s+/, ${$subfullIPdatabase}{$IP} );
		shift @array;
		shift @array;
		print "$IP	|	$IPall{$IP}	|	@array	\n";
    }

    elsif ( ${$subfullDHCP}{$IP} ) {
		print "$IP	|	$IPall{$IP}	|	${$subfullDHCP}{$IP}	\n";
		}

    }

}
#####################
#####################

#### Subroutine for creating IP global HTML report with file 

sub IPglobREPORT {

    # IPglobREPORT($reports_folder,\@total,\%uniqIP,\%fullDHCP,\%fullIPdatabase)

    my $subreport_folder = shift;
    my $subtotal         = shift;
    my $subuniqIP        = shift;
	my $subfullDHCP =shift;
	my $subfullIPdatabase=shift;
	
    my %IPsize   = %{$subuniqIP};
    my %IPdomain = %{$subuniqIP};
    my %IPall    = %{$subuniqIP};
    my %uniqdomains;
	my $identificator;
	

    foreach ( @{$subtotal} ) {

        my @array = split( /\s+/, $_ );

        $IPsize{ $array[1] } = $IPsize{ $array[1] } + $array[2];
        $IPall{ $array[1] }++;

        if ( $array[3] =~ m{(^.*?\.\w*\/)} ) {
            $uniqdomains{$1} = 0;
        }

    }

    foreach my $IP ( keys(%uniqIP) ) {
        my @oneIPtotal;
        GEToneIPtotal( $IP, $subtotal, \@oneIPtotal );
        my %oneIPdomain;
        GEToneIPdomain( $IP, \@oneIPtotal, \%oneIPdomain );

        $IPdomain{$IP} = keys(%oneIPdomain);

    }
    my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime( time() );
    my $timestamp =
        ( $year + 1900 ) . "-"
      . sprintf( "%02d", $mon + 1 ) . "-"
      . sprintf( "%02d", $mday ) . "T"
      . sprintf( "%02d", $hour ) 
      . sprintf( "%02d", $min ) 
      . sprintf( "%02d", $sec );

    open FH, ">$subreport_folder". "IPreport_"  . $timestamp . ".html" or  die "Couldn't open html global IP report '$subreport_folder IPreport.$timestamp.html'\n";


my $TimeRange=GetTimeRange($subtotal);
$timestamp =
        ( $year + 1900 ) . "-"
      . sprintf( "%02d", $mon + 1 ) . "-"
      . sprintf( "%02d", $mday ) . " "
      . sprintf( "%02d", $hour ) . ":"
      . sprintf( "%02d", $min ) . ":"
      . sprintf( "%02d", $sec );

    print FH
"\<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\"\>
\<html xmlns=\"http://www.w3.org/1999/xhtml\"\>
	\<head\>
		\<meta name=\"Global_IP\" content=\"Global HTML report fo IP addresses\" /\>
		\<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" /\>
		\<title\>Global report for all IP adresses \</title\>
	\</head\>
	\<body\>
		\<h4\> Global IP report generated at $timestamp with data time period '$TimeRange' \</h4\>
		\<h2\> IP and size of transfered data \</h2\>
		\<table border=\"1\"\>
			\<tr\>
				\<th\> IP \</th\>
				\<th\> Size \</th\>
				\<th\> Identification\</th\>
			\</tr\>";

    foreach my $IP ( sort { $IPsize{$b} <=> $IPsize{$a} } keys(%IPsize) ) {
			
			if ( ${$subfullIPdatabase}{$IP} ) {

        my @array = split( /\s+/, ${$subfullIPdatabase}{$IP} );
		shift @array;
		shift @array;
		$identificator="@array";
    }

    elsif ( ${$subfullDHCP}{$IP} ) {
		$identificator="${$subfullDHCP}{$IP}";
		}
	
	
        my ( $size, $unit ) = sizecon( $IPsize{$IP} );
        print FH"
			\<tr\>
				\<td\>  $IP \</td\>
				\<td\>$size $unit\</td\>
				\<td\>$identificator\</td\>
			\</tr\>";
    }
    print FH"
		\</table\>
		\<h2\> IP and sum of unique domains \</h2\>
		\<table border=\"1\"\>
			\<tr\>
				\<th\> IP \</th\>
				\<th\> Domains sum \</th\>
				\<th\> Identification \</th\>
			\</tr\>";

    foreach my $IP ( sort { $IPdomain{$b} <=> $IPdomain{$a} } keys(%IPdomain) )
    {
				if ( ${$subfullIPdatabase}{$IP} ) {

        my @array = split( /\s+/, ${$subfullIPdatabase}{$IP} );
		shift @array;
		shift @array;
		$identificator="@array";
    }

    elsif ( ${$subfullDHCP}{$IP} ) {
		$identificator="${$subfullDHCP}{$IP}";
		}
	
	
	
        print FH"
			\<tr\>
				\<td\>  $IP \</td\>
				\<td\>$IPdomain{$IP}\</td\>
				\<td\>$identificator\</td\>
			\</tr\>";
    }
    print FH"
		\</table\>
		\<h2\> IP and sum of all HTTP request \</h2\>
		\<table border=\"1\"\>
			\<tr\>
				\<th\> IP \</th\>
				\<th\> HTTP requests sum \</th\>
				\<th\> Identification \</th\>
			\</tr\>";

    foreach my $IP ( sort { $IPall{$b} <=> $IPall{$a} } keys(%IPall) ) {
	
		if ( ${$subfullIPdatabase}{$IP} ) {

        my @array = split( /\s+/, ${$subfullIPdatabase}{$IP} );
		shift @array;
		shift @array;
		$identificator="@array";
    }

    elsif ( ${$subfullDHCP}{$IP} ) {
		$identificator="${$subfullDHCP}{$IP}";
		}
	
        print FH"
			\<tr\>
				\<td\>  $IP \</td\>
				\<td\> $IPall{$IP} \</td\>
				\<td\> $identificator \</td\>
			\</tr\>";
    }
    print FH"
		\</table\>
	\</body\>
\</html\>";

    close FH
      or die "Unable to close '$subreport_folder IPreport.$timestamp.html' \n";


}
#####################
#####################

#### Subroutine for creating global domain report without HTML file

sub DOMAINglobNOreport {

    # DOMAINglobNOreport(\@total)

    my $subtotal = shift;
    my %domainSUM;
    my %sizeSUM;

    foreach ( @{$subtotal} ) {

        my @array = split( /\s+/, $_ );

        if ( $array[3] =~ m{(^.*?\.\w*\/)} ) {
            $domainSUM{$1}++;
            if ( !$sizeSUM{$1} ) {
                $sizeSUM{$1} = 0;
            }
            $sizeSUM{$1} = $sizeSUM{$1} + $array[2];
        }

    }

    print "\n	Size	|	Domain 	\n-------------------------------------------------------------------\n";
    foreach my $domain ( sort { $sizeSUM{$b} <=> $sizeSUM{$a} } keys(%sizeSUM) )
    {
        my ( $size, $unit ) = sizecon( $sizeSUM{$domain} );
        print "  $size $unit	|	$domain\n";
    }
    print "\n\n\n\n	Domain requests sum	|	Domain	\n-------------------------------------------------------------------\n";
    foreach
      my $domain ( sort { $domainSUM{$b} <=> $domainSUM{$a} } keys(%domainSUM) )
    {
        print "  $domainSUM{$domain}	|	$domain\n";
    }

}
#####################
#####################

#### Subroutine for creation of global domain report with HTML file

sub DOMAINglobREPORT {

    # DOMAINglobREPORT($reports_folder,\@total)

    my $subreport_folder = shift;
    my $subtotal         = shift;
    my %domainSUM;
    my %sizeSUM;

    foreach ( @{$subtotal} ) {

        my @array = split( /\s+/, $_ );

        if ( $array[3] =~ m{(^.*?\.\w*\/)} ) {
            $domainSUM{$1}++;
            if ( !$sizeSUM{$1} ) {
                $sizeSUM{$1} = 0;
            }
            $sizeSUM{$1} = $sizeSUM{$1} + $array[2];
        }

    }

    my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime( time() );
    my $timestamp =
        ( $year + 1900 ) . "-"
      . sprintf( "%02d", $mon + 1 ) . "-"
      . sprintf( "%02d", $mday ) . "T"
      . sprintf( "%02d", $hour )
      . sprintf( "%02d", $min )
      . sprintf( "%02d", $sec );
	  
	  

    open FH, ">$subreport_folder" . "DOMAINreport_" . $timestamp . ".html"
      or die "Couldn't open html global DOMAIN report '$subreport_folder DOMAINreport.$timestamp.html'\n";


my $TimeRange=GetTimeRange($subtotal);
$timestamp =
        ( $year + 1900 ) . "-"
      . sprintf( "%02d", $mon + 1 ) . "-"
      . sprintf( "%02d", $mday ) . " "
      . sprintf( "%02d", $hour ) . ":"
      . sprintf( "%02d", $min ) . ":"
      . sprintf( "%02d", $sec );
		
    print FH
"\<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\"\>
\<html xmlns=\"http://www.w3.org/1999/xhtml\"\>
	\<head\>
		\<meta name=\"Global_domain\" content=\"Global HTML report for domains\" /\>
		\<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" /\>
		\<title\>Global report for all Domains \</title\>
	\</head\>
	\<body\>
		\<h4\> Global IP report generated at $timestamp with data time period '$TimeRange' \</h4\>
		\<h2\> Domain and size of transfered data \</h2\>
		\<table border=\"1\"\>
			\<tr\>
				\<th\> Domain \</th\>
				\<th\> Size \</th\>
			\</tr\>";

    foreach my $domain ( sort { $sizeSUM{$b} <=> $sizeSUM{$a} } keys(%sizeSUM) )
    {
        my ( $size, $unit ) = sizecon( $sizeSUM{$domain} );
        print FH"
			\<tr\>
				\<td\>  $domain \</td\>
				\<td\>$size $unit\</td\>
			\</tr\>";
    }
    print FH"
		\</table\>
		\<h2\> IP and sum of unique domains \</h2\>
		\<table border=\"1\"\>
			\<tr\>
				\<th\> Domain \</th\>
				\<th\> Domains sum \</th\>
			\</tr\>";

    foreach
      my $domain ( sort { $domainSUM{$b} <=> $domainSUM{$a} } keys(%domainSUM) )
    {
        print FH"
			\<tr\>
				\<td\>  $domain \</td\>
				\<td\>$domainSUM{$domain}\</td\>
			\</tr\>";
    }
    print FH"
		\</table\>
	\</body\>
\</html\>";

    close FH
      or die "Unable to close '$subreport_folder DOMAINreport.$timestamp.html' \n";


}
#####################
#####################

#### Subroutine for quick report about desired domain or web adress, output only to screen

sub URLreport {

    # URLreport($reports_folder,$url,\@total,\%uniqIP,\%fullDHCP,\%fullIPdatabase)

    my $subreport_folder = shift;
    my $suburl           = shift;
    my $subtotal         = shift;
    my $subuniqIP        = shift;
	my $subfullDHCP =shift;
	my $subfullIPdatabase=shift;
    my %urlSUM           = %{$subuniqIP};

    foreach ( @{$subtotal} ) {

        my @array = split( /\s+/, $_ );

        if ( $array[3] =~ m{(^.*$suburl.*)} ) {
            $urlSUM{ $array[1] }++;
        }

    }

    print "\n	IP	|	Sum 	|	Identification \n------------------------------------------------------------\n";
    foreach my $IP ( sort { $urlSUM{$b} <=> $urlSUM{$a} } keys(%urlSUM) ) {
	
	if ($urlSUM{$IP}!=0)
		{
	
	if ( ${$subfullIPdatabase}{$IP} ) {

        my @array = split( /\s+/, ${$subfullIPdatabase}{$IP} );
		shift @array;
		shift @array;
		
        print "$IP	|	$urlSUM{$IP}	|	@array \n";

    }

    elsif ( ${$subfullDHCP}{$IP} ) {
		print "$IP	|	$urlSUM{$IP}	|	${$subfullDHCP}{$IP} \n";
		}

		}
	}

}



#====================================================================
################# END OF SUBROUTINES DEFINITION

